'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _signals = require('./signals');

var _constants = require('./constants');

var _options = require('./options');

var _utils = require('./utils');

var _resolvers = require('./resolvers');

var _resolvers2 = _interopRequireDefault(_resolvers);

var _createRegistrationAPI = require('./createRegistrationAPI');

var _createRegistrationAPI2 = _interopRequireDefault(_createRegistrationAPI);

var Context = (function () {
	function Context(conf, logger, mappings, signalRelease) {
		_classCallCheck(this, Context);

		var api;

		if (typeof conf !== 'function') {
			throw new Error('Invalid container creation, missing contribution function');
		}

		this._mappings = mappings || {};
		this._resolvers = _resolvers2['default'];
		this._logger = logger;
		this._resolving = {};
		this._pending = [];
		this._children = {};
		this._signalRelease = signalRelease || _utils.noop;

		this._bind(_constants.CONTEXT_ALIAS, this, _options.VALUE, []);
		api = (0, _createRegistrationAPI2['default'])(this._bind.bind(this));
		conf(api.map);
		api.done();
	}

	_createClass(Context, [{
		key: 'get',
		value: function get(alias) {
			var _this = this;

			return this._logger.log('resolving \'' + alias + '\'', function () {
				if (_this._resolving[alias]) {
					throw new Error('Circular dependency detected while resolving \'' + alias + '\'');
				}
				if (!(alias in _this._mappings)) {
					throw new Error('\'' + alias + '\' is not available. Has it ever been registered?.');
				}

				_this._resolving[alias] = true;
				try {
					return _this._mappings[alias](_signals.ACQUIRE);
				} catch (err) {
					err.message = 'Failed while resolving \'' + alias + '\' due to:\n\t' + err.message;
					throw err;
				} finally {
					_this._resolving[alias] = false;
				}
			});
		}
	}, {
		key: 'using',
		value: function using(transientsDeps) {
			var _this2 = this;

			return {
				get: function get(alias) {
					var dep;

					for (dep in transientsDeps) {
						_this2._bind(dep, transientsDeps[dep], _options.VALUE, []);
					}
					_this2.get(alias);
					for (dep in transientsDeps) {
						_this2._unbind(dep);
					}
				}
			};
		}
	}, {
		key: 'release',
		value: function release(alias) {
			try {
				this._mappings[alias](_signals.RELEASE);
			} catch (err) {
				err.message = 'Failed while releasing \'' + alias + '\' due to:\n\t' + err.message;
				throw err;
			}
		}
	}, {
		key: 'createChild',
		value: function createChild(conf) {
			var id = Object.keys(this._children).length + 1;
			var child = new Context(conf, this._logger, Object.create(this._mappings), this._releaseChild.bind(this, id));

			this._children[id] = child;
			return child;
		}
	}, {
		key: 'dispose',
		value: function dispose() {
			this._disposeChildren();
			this._disposeInstances();
			this._signalRelease();
			this._signalRelease = _utils.noop;
		}
	}, {
		key: '_disposeChildren',
		value: function _disposeChildren() {
			var children = this._children;
			var id;

			for (id in children) {
				/* istanbul ignore else  */
				if (children.hasOwnProperty(id)) {
					children[id].dispose();
					this._releaseChild(id);
				}
			}
		}
	}, {
		key: '_releaseChild',
		value: function _releaseChild(id) {
			delete this._children[id];
		}
	}, {
		key: '_disposeInstances',
		value: function _disposeInstances() {
			var mappings = this._mappings;
			var alias;

			for (alias in mappings) {
				/* istanbul ignore else  */
				if (mappings.hasOwnProperty(alias)) {
					try {
						mappings[alias](_signals.DISPOSE);
					} catch (err) {
						err.message = 'Failed while disposing \'' + alias + '\' due to:\n\t' + err.message;
						throw err;
					}
					delete mappings[alias];
				}
			}
		}
	}, {
		key: '_bind',
		value: function _bind(alias, value, type, deps) {
			if (!(type in this._resolvers)) {
				throw new Error('Invalid flags combination. See documentation for valid flags combinations.');
			}
			this._mappings[alias] = this._resolvers[type].call(null, value, this._resolve.bind(this, deps), this._release.bind(this, deps));
			if ((0, _utils.isEagerSingleton)(type)) {
				this.get(alias);
			}
		}
	}, {
		key: '_unbind',
		value: function _unbind(alias) {
			this._mappings[alias](_signals.DISPOSE);
			delete this._mappings[alias];
		}
	}, {
		key: '_release',
		value: function _release(deps) {
			var _this3 = this;

			return deps.forEach(function (dep) {
				_this3.release(dep);
			});
		}
	}, {
		key: '_resolve',
		value: function _resolve(deps) {
			var _this4 = this;

			return deps.map(function (dep) {
				return _this4.get(dep);
			});
		}
	}]);

	return Context;
})();

exports['default'] = Context;
module.exports = exports['default'];