'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports['default'] = singletonify;

var _signals = require('./signals');

function singletonify(create, LAZY) {

	LAZY = LAZY || false;

	return function (value, resolveDeps, releaseDeps) {

		var handlers = {};
		var count = 0;
		var instance;

		function dispose() {
			if (typeof instance.dispose === 'function') {
				instance.dispose();
			}
			instance = undefined;
			count = 0;
		}

		handlers[_signals.ACQUIRE] = function (value) {
			if (typeof instance === 'undefined') {
				instance = create.call(this, value, resolveDeps());
			}
			count++;
			return instance;
		};

		handlers[_signals.RELEASE] = function (value) {
			count--;
			if (count == 0 && !LAZY) {
				releaseDeps();
				dispose();
			}
		};

		handlers[_signals.DISPOSE] = function (value) {
			if (instance) {
				releaseDeps();
				dispose();
			}
		};

		return function (signal) {
			return handlers[signal].call(this, value);
		};
	};
}

module.exports = exports['default'];