'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports.identity = identity;
exports.leftCurryTwice = leftCurryTwice;
exports.compose = compose;
exports.resolveDeps = resolveDeps;
exports.invoke = invoke;
exports.partial = partial;
exports.generateMask = generateMask;
exports.isEagerSingleton = isEagerSingleton;
exports.noop = noop;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _instanciate = require('./instanciate');

var _instanciate2 = _interopRequireDefault(_instanciate);

var _singletonify = require('./singletonify');

var _singletonify2 = _interopRequireDefault(_singletonify);

var _extractImplicitDependencies = require('./extractImplicitDependencies');

var _extractImplicitDependencies2 = _interopRequireDefault(_extractImplicitDependencies);

var _options = require('./options');

var EAGER_SINGLETON_PROVIDER = generateMask([_options.EAGER, _options.SINGLETON, _options.PROVIDER]);
var EAGER_SINGLETON_CONSTRUCTOR = generateMask([_options.EAGER, _options.SINGLETON, _options.CONSTRUCTOR]);

function identity(value) {
	return value;
}

function leftCurryTwice(fn) {
	return function firstCurry() {
		for (var _len = arguments.length, firstArgs = Array(_len), _key = 0; _key < _len; _key++) {
			firstArgs[_key] = arguments[_key];
		}

		return function secondCurry() {
			for (var _len2 = arguments.length, secondArgs = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
				secondArgs[_key2] = arguments[_key2];
			}

			return fn.apply(null, firstArgs.concat(secondArgs));
		};
	};
}

function compose() {
	for (var _len3 = arguments.length, funcs = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
		funcs[_key3] = arguments[_key3];
	}

	return function () {
		for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
			args[_key4] = arguments[_key4];
		}

		var i;

		for (i = funcs.length - 1; i >= 0; i--) {
			args = [funcs[i].apply(this, args)];
		}
		return args[0];
	};
}

function resolveDeps(fn) {
	return function (value, resolve, release, signal) {
		return fn.call(this, value, resolve());
	};
}

function invoke(fn, deps) {
	return fn.apply(null, deps);
}

function partial(func, boundArgs) {
	return function () {
		for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
			args[_key5] = arguments[_key5];
		}

		return func.apply(this, boundArgs.concat(args));
	};
}

function generateMask(flags) {
	return flags.reduce(function (prev, curr) {
		return prev | curr;
	}, 0);
}

function isEagerSingleton(type) {
	return [EAGER_SINGLETON_PROVIDER, EAGER_SINGLETON_CONSTRUCTOR].indexOf(type) > -1;
}

function noop() {}

exports.instanciate = _instanciate2['default'];
exports.singletonify = _singletonify2['default'];
exports.extractImplicitDependencies = _extractImplicitDependencies2['default'];